// VinciaFunctions.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2013 P. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for VINCIA's Functions class.
  
#ifndef VINCIAFUNCTIONS_H
#define VINCIAFUNCTIONS_H

#include "Basics.h"                   // Vec4
#include "Event.h"                    // Particle
#include "VinciaCommon.h"             // Common

using namespace Pythia8;

namespace Vincia {

  class Functions {

  public: 

    // Constructor
    Functions() {};

    // *********************************************************************
    // Auxiliary Hardcoded Matrix Elements and Antenna Functions
    // *********************************************************************
    
    // Eikonal (without color or coupling factors)
    double Eikonal(Vec4 a, Vec4 r, Vec4 b) {
      double sar = m2(a+r);
      double srb = m2(r+b);
      double sab = m2(a+b);
      return 2*sab/(sar*srb);
    }
    double Eikonal(Particle a, Particle r, Particle b) {
      return Eikonal(a.p(), r.p(), b.p());
    }
    double Eikonal(Particle* a, Particle* r, Particle* b) {
      return Eikonal(a->p(), r->p(), b->p());
    }
    double Eikonal(double yij, double yjk) {
      return 2.*(1.-yij-yjk)/yij/yjk;
    }

    // Helicity-dependent Altarelli-Parisi kernels (mu = m/Q)
    // Note: Pg2gg is written with standard factor 2 normalization convention
    double Pg2gg(double z, int hA=9, int hB=9, int hC=9); 
    double Pg2qq(double z, int hA=9, int hB=9, int hC=9, double mu=0.); 
    double Pq2qg(double z, int hA=9, int hB=9, int hC=9, double mu=0.); 
    double Pq2gq(double z, int hA=9, int hB=9, int hC=9, double mu=0.) {
      return Pq2qg(1-z,mu,hA,hC,hB);
    }

    // Wrappers to get unpolarized massive Altarelli-Pariis kernels (mu = m/Q)
    double Pg2qq(double z, double mu) { return Pg2qq(z,9,9,9,mu); }
    double Pq2qg(double z, double mu) { return Pq2qg(z,9,9,9,mu); }
    double Pq2gq(double z, double mu) { return Pq2gq(z,9,9,9,mu); }

    // Altarelli-Parisi kernels with linear in/out polarizations for gluons
    //   pol = +1 for in-plane, -1 for out-of-plane
    double Pg2ggLin(double z, int polA=9, int polB=9, int polC=9); 
    double Pg2qqLin(double z, int polA=9, int hB=9, int hC=9, double mu=0.); 
    double Pq2qgLin(double z, int hA=9, int hB=9, int polC=9,double mu=0.); 
    double Pq2gqLin(double z, int hA=9, int polB=9, int hC=9,double mu=0.) {
      return Pq2qgLin(1-z,mu,hA,hC,polB);
    }

    // Z -> 3-parton antenna functions (= tree-level GGG antennae)
    double A30(vector<Vec4>& pn, int a, int r, int b);
    // Z -> 4-parton antenna functions (= tree-level GGG antennae)
    double A40(vector<Vec4>& pn, int a, int b, int c, int d);
    double A40aux(vector<Vec4>& pn, int a,int b,int c,int d);
    double B40(vector<Vec4>& pn, int a, int b, int c, int d);
    double B40aux(vector<Vec4>& pn, int a, int b, int c, int d);

  private:

  };

}

#endif
